
import java.io.*;
import javax.vecmath.*;
import org.autochthonous.graphics.rt.tools.io.*;

import org.autochthonous.graphics.rt.*;
import org.autochthonous.graphics.rt.objects.*;

/**
 * a class describing a kummer surface - if you need to do a role
 * your own algebraic surface, this is how it's done...
 */
public class KummerSurface
    extends Algebraic
{
    RayPoly xSqd = new RayPoly(2);
    RayPoly ySqd = new RayPoly(2);
    RayPoly zSqd = new RayPoly(2);

    RayPoly yzSqd = new RayPoly(4);
    RayPoly zxSqd = new RayPoly(4);
    RayPoly xySqd = new RayPoly(4);

    RayPoly poly = new RayPoly(4);

    KummerSurface()
    {
        super(4);
    }

    public static Algebraic getInstance(
        RMLReader       in)
        throws IOException
    {
        KummerSurface   a = new KummerSurface();

        a.readObject(in);

        return a;
    }

    /**
     *  x^{4} + y^{4} + z^{4} + 1 - x^{2} - y^{2} - z^{2} - y^{2}z^{2} - z^{2}x^{2} - x^{2}y^{2} = 0
     */
    protected RayPoly getPolynomial(
        Ray r)
    {
        xSqd.zero();
        xSqd.addXpow(r, 2);

        ySqd.zero();
        ySqd.addYpow(r, 2);

        zSqd.zero();
        zSqd.addZpow(r, 2);

        yzSqd.set(ySqd);
        yzSqd.multiply(zSqd);

        zxSqd.set(zSqd);
        zxSqd.multiply(xSqd);

        xySqd.set(xSqd);
        xySqd.multiply(ySqd);

        poly.zero();
        poly.addXpow(r, 4);
        poly.addYpow(r, 4);
        poly.addZpow(r, 4);
        poly.add(1);
        poly.subtract(xSqd);
        poly.subtract(ySqd);
        poly.subtract(zSqd);
        poly.subtract(yzSqd);
        poly.subtract(zxSqd);
        poly.subtract(xySqd);

        return poly;
    }

    /**
     * Calculate the normal components,
     * this is simply the partial derivitives in x, y, and z.
     *  <ul>
     *  <li>4x^{3} - 2x - 2xz^{2} - 2xy^{2}
     *  <li>4y^{3} - 2y - 2yz^{2} - 2yx^{2}
     *  <li>4z^{3} - 2z - 2zy^{2} - 2zx^{2}
     *  </ul>
     */
    public void getN(
        Hit         hit,
        Vector3f    n)
    {
        Point3f     loc = hit.oP();

        float x2 = loc.x * loc.x;
        float y2 = loc.y * loc.y;
        float z2 = loc.z * loc.z;

        n.set(
                loc.x * ((2 * x2) - (1 + z2 + y2)),
                loc.y * ((2 * y2) - (1 + z2 + x2)),
                loc.z * ((2 * z2) - (1 + y2 + x2)));
    }
}
