package org.autochthonous.graphics.rt.models;

import java.io.*;

import java.awt.*;
import java.awt.image.*;

import javax.vecmath.*;

import org.autochthonous.graphics.rt.util.*;
import org.autochthonous.graphics.rt.tools.io.*;
import org.autochthonous.graphics.rt.objects.Heightfield;

public class ImageHeightfield
    extends Heightfield
{
    private ImageHeightfield()
    {
    }

    public static Heightfield getInstance(
        RMLReader       in)
        throws IOException
    {
        ImageHeightfield b = new ImageHeightfield();

        b.readObject(in);

        return b;
    }

    public void setTile(
        String          tileName)
        throws IOException
    {
        if (tileName == null)
        {
            throw new IOException("null tile name passed in");
        }

        Image           tileIm = ImageUtil.loadImage(tileName);
        PixelGrabber    pg = new PixelGrabber(tileIm, 0, 0, -1, -1, true);

        try
        {
            if (pg.grabPixels())
            {
                this.pixVals = (int[])pg.getPixels();
                this.pixWidth = pg.getWidth();
                this.pixHeight = pg.getHeight();
            }
            else
            {
                throw new RuntimeException("can't grab pixels");
            }
        }
        catch (InterruptedException e)
        {
            throw new RuntimeException(e.toString());
        }
    }

    private static float getHeight(
        int pix)
    {
        pix &= 0xffffff;

        float   r = ((pix >> 16) & 0xff) / 255.0f;
        float   g = ((pix >> 8) & 0xff) / 255.0f;
        float   b = (pix & 0xff) / 255.0f;

        return (float)Math.sqrt(r * 0.3 + g * 0.59 + b * 0.11);
    }

    private static float[][] zValues(
        String  name)
        throws IOException
    {
        Image           elevIm = ImageUtil.loadImage(name);
        PixelGrabber    pg = new PixelGrabber(elevIm, 0, 0, -1, -1, true);
        int[]           pix;
        int             xSize, ySize;

        try
        {
            if (pg.grabPixels())
            {
                pix = (int[])pg.getPixels();
                xSize = pg.getWidth();
                ySize = pg.getHeight();
            }
            else
            {
                throw new RuntimeException("can't grab pixels");
            }
        }
        catch (InterruptedException e)
        {
            throw new RuntimeException(e.toString());
        }

        float[][]   zVals = new float[ySize][];

        for (int i = 0; i != ySize; i++)
        {
            int off = i * ySize;
            float[] row = zVals[i] = new float[xSize];

            for (int j = 0; j != xSize; j++)
            {
                int pixel = pix[off + j];

                if (((pixel >> 24) & 0xff) == 0xff)
                {
                    row[j] = getHeight(pixel);
                }
                else
                {
                    float   fact = ((pixel >> 24) & 0xff) / 255.0f;
                    row[j] = getHeight(pixel) * fact;
                }
            }
        }

        return zVals;
    }

    public ImageHeightfield(
        Point3f         p1,
        Point3f         p2,
        String          name)
        throws IOException
    {
        super(p1, p2, zValues(name));
    }

    public ImageHeightfield(
        Point3f         p1,
        Point3f         p2,
        String          name,
        String          tileName)
        throws IOException
    {
        super(p1, p2, zValues(name));

        setTile(tileName);
    }

    protected void processParameters(
        ParametersMap    params)
    {
        super.processParameters(params);

        String  name = params.getString("name", null);
        String  tileName = params.getString("tile", null);

        try
        {
            initZ(zValues(name));
            if (tileName != null)
            {
                setTile(tileName);
            }
        }
        catch (IOException e)
        {
            throw new RuntimeException(e.toString());
        }
    }
}
