package org.autochthonous.graphics.rt.util;

import java.io.*;

import java.awt.*;
import java.awt.image.*;

import java.util.*;

public class ImageUtil
{
	static final byte LF = 0x0A;
	static final byte CR = 0x0D;

	private static String readLine(BufferedInputStream in)
        throws IOException
	{
		String s = new String();
		int b;

		do {
			b = in.read();
			if (b != LF && b != CR) s = s + (char)b;
			if (b == CR) {
				in.mark(1);	// this will always succeed for buffered input streams
				b = in.read();
				if (b != LF) {
					// we've found a lone CR, reset buffer to before char of next line
					in.reset();
					}
				}
			} while (b != LF && b != CR);

		return s;
	}

    public static Image loadImage(
        String  name)
        throws IOException
    {
        Image   im = null;

        if (name == null)
        {
            throw new IllegalArgumentException("No image name specified in loadImage.");
        }

        Toolkit kit = Toolkit.getDefaultToolkit();

        if (name.endsWith(".pix"))
        {
            BufferedInputStream  rIn = new BufferedInputStream(new FileInputStream(name));
            String  type = readLine(rIn);
            String  dimensions = readLine(rIn);
            int[]   pix;
            int     width, height;

            if (type.equals("RGB"))
            {
                StringTokenizer st = new StringTokenizer(dimensions, " ");

                width = Integer.parseInt(st.nextToken());
                height = Integer.parseInt(st.nextToken());

                int depth = Integer.parseInt(st.nextToken());

                pix = new int[width * height];

                for (int i = 0; i != pix.length; i++)
                {
                    pix[i] = (255 << 24) | (rIn.read() << 16) | (rIn.read() << 8) | (rIn.read());
                }
            }
            else
            {
                throw new IOException("Unknown image type");
            }

            im = kit.createImage(new MemoryImageSource(width, height, pix, 0, width));
        }
        else
        {
            im = kit.getImage(name);
        }

        return im;
    }
}
